@@ -11,6 +11,7 @@ from tg.configuration.app_config import TGConfigError
 from tg.configuration.auth import _AuthenticationForgerPlugin
 from tg.configuration.auth.metadata import _AuthMetadataAuthenticator
 from tg.configuration.utils import coerce_config
+from tg.configuration import milestones
 from paste.deploy.converters import asint
 
 import tg.i18n
@@ -22,8 +23,10 @@ from tg.wsgiapp import TGApp
 from tg._compat import PY3
 
 def setup():
+    milestones._reset_all()
     setup_session_dir()
 def teardown():
+    milestones._reset_all()
     teardown_session_dir()
 
 class PackageWithModel:
@@ -138,6 +141,8 @@ class TestAppConfig:
         self.fake_package = PackageWithModel
 
     def setup(self):
+        milestones._reset_all()
+
         self.config = AppConfig()
         # set up some required paths and config settings
         # FIXME: these seem to be needed so that
@@ -147,20 +152,20 @@ class TestAppConfig:
         self.config.package = self.fake_package
         self.config['paths']['root'] = 'test'
         self.config['paths']['controllers'] = 'test.controllers'
-        self.config.init_config({'cache_dir':'/tmp'}, {})
+        self.config._init_config({'cache_dir':'/tmp'}, {})
 
         config['paths']['static_files'] = "test"
         config["tg.app_globals"] = Bunch()
         config["use_sqlalchemy"] = False
         config["global_conf"] = Bunch()
         config["package"] = "test"
-        config["call_on_shutdown"] = "foo"
         config["render_functions"] = Bunch()
         config['beaker.session.secret'] = 'some_secret'
 
     def teardown(self):
         #This is here to avoid that other tests keep using the forced controller
         config.pop('tg.root_controller', None)
+        milestones._reset_all()
 
     def test_get_root(self):
         current_root_module = self.config['paths']['root']
@@ -171,7 +176,7 @@ class TestAppConfig:
 
     def test_lang_can_be_changed_by_ini(self):
         conf = AppConfig(minimal=True)
-        conf.init_config({'lang':'ru'}, {})
+        conf._init_config({'lang':'ru'}, {})
         assert config['lang'] == 'ru'
 
     def test_create_minimal_app(self):
@@ -214,21 +219,21 @@ class TestAppConfig:
         def func():
             a = 7
         self.config.call_on_startup = [func]
-        self.config.setup_startup_and_shutdown()
+        self.config._setup_startup_and_shutdown()
 
     def test_setup_startup_and_shutdown_callable_startup_with_exception(self):
         def func():
             raise Exception
         self.config.call_on_startup = [func]
-        self.config.setup_startup_and_shutdown()
+        self.config._setup_startup_and_shutdown()
 
     def test_setup_startup_and_shutdown_startup_not_callable(self):
         self.config.call_on_startup = ['not callable']
-        self.config.setup_startup_and_shutdown()
+        self.config._setup_startup_and_shutdown()
 
     def test_setup_startup_and_shutdown_shutdown_not_callable(self):
         self.config.call_on_shutdown = ['not callable']
-        self.config.setup_startup_and_shutdown()
+        self.config._setup_startup_and_shutdown()
 
     @raises(AtExitTestException)
     def test_setup_startup_and_shutdown_shutdown_callable(self):
@@ -236,7 +241,7 @@ class TestAppConfig:
             raise AtExitTestException()
 
         self.config.call_on_shutdown = [func]
-        self.config.setup_startup_and_shutdown()
+        self.config._setup_startup_and_shutdown()
         atexit._run_exitfuncs()
 
     def test_setup_helpers_and_globals(self):
@@ -598,11 +603,13 @@ class TestAppConfig:
         def dummy(*args):
             pass
 
+        milestones.config_ready._reset()
         self.config.register_hook('startup', dummy)
         self.config.register_hook('shutdown', dummy)
         self.config.register_hook('controller_wrapper', dummy)
         for hook_name in self.config.hooks.keys():
             self.config.register_hook(hook_name, dummy)
+        milestones.config_ready.reach()
 
         for hooks in self.config.hooks.values():
             assert hooks
@@ -620,7 +627,7 @@ class TestAppConfig:
     def test_controler_wrapper_setup(self):
         orig_caller = self.config.controller_caller
         self.config.controller_wrappers = []
-        self.config.setup_controller_wrappers()
+        self.config._setup_controller_wrappers()
         assert config['controller_caller'] == orig_caller
 
         def controller_wrapper(app_config, caller):
@@ -630,9 +637,90 @@ class TestAppConfig:
 
         orig_caller = self.config.controller_caller
         self.config.controller_wrappers = [controller_wrapper]
-        self.config.setup_controller_wrappers()
+        self.config._setup_controller_wrappers()
         assert config['controller_caller'].__name__ == controller_wrapper(self.config, orig_caller).__name__
 
+    def test_global_controller_wrapper(self):
+        milestones._reset_all()
+
+        class RootController(TGController):
+            @expose()
+            def test(self):
+                return 'HI!'
+
+        wrapper_has_been_visited = []
+        def controller_wrapper(app_config, caller):
+            def call(*args, **kw):
+                wrapper_has_been_visited.append(True)
+                return caller(*args, **kw)
+            return call
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+        conf.register_hook('controller_wrapper', controller_wrapper)
+        conf.package = PackageWithModel()
+        app = conf.make_wsgi_app()
+        app = TestApp(app)
+
+        assert 'HI!' in app.get('/test')
+        assert wrapper_has_been_visited[0] is True
+
+    def test_dedicated_controller_wrapper(self):
+        milestones._reset_all()
+
+        class RootController(TGController):
+            @expose()
+            def test(self):
+                return 'HI!'
+
+        wrapper_has_been_visited = []
+        def controller_wrapper(app_config, caller):
+            def call(*args, **kw):
+                wrapper_has_been_visited.append(True)
+                return caller(*args, **kw)
+            return call
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+        tg.hooks.wrap_controller(controller_wrapper, controller=RootController.test)
+        conf.package = PackageWithModel()
+        app = conf.make_wsgi_app()
+        app = TestApp(app)
+
+        assert 'HI!' in app.get('/test')
+        assert wrapper_has_been_visited[0] is True
+
+    def test_mixed_controller_wrapper(self):
+        milestones._reset_all()
+
+        class RootController(TGController):
+            @expose()
+            def test(self):
+                return 'HI!'
+
+        app_wrapper_has_been_visited = []
+        def app_controller_wrapper(app_config, caller):
+            def call(*args, **kw):
+                app_wrapper_has_been_visited.append(True)
+                return caller(*args, **kw)
+            return call
+
+        wrapper_has_been_visited = []
+        def controller_wrapper(app_config, caller):
+            def call(*args, **kw):
+                wrapper_has_been_visited.append(True)
+                return caller(*args, **kw)
+            return call
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+        tg.hooks.wrap_controller(app_controller_wrapper)
+        tg.hooks.wrap_controller(controller_wrapper, controller=RootController.test)
+        conf.package = PackageWithModel()
+        app = conf.make_wsgi_app()
+        app = TestApp(app)
+
+        assert 'HI!' in app.get('/test')
+        assert wrapper_has_been_visited[0] is True
+        assert app_wrapper_has_been_visited[0] is True
+
     def test_application_wrapper_setup(self):
         class RootController(TGController):
             @expose()
@@ -665,17 +753,34 @@ class TestAppConfig:
             pass
         class AppWrapper4:
             pass
+        class AppWrapper5:
+            pass
 
         conf = AppConfig(minimal=True)
         conf.register_wrapper(AppWrapper2)
+        conf.register_wrapper(AppWrapper4, after=AppWrapper3)
         conf.register_wrapper(AppWrapper3)
         conf.register_wrapper(AppWrapper1, after=False)
-        conf.register_wrapper(AppWrapper4, after=AppWrapper3)
+        conf.register_wrapper(AppWrapper5, after=AppWrapper3)
+        milestones.environment_loaded.reach()
 
         assert conf.application_wrappers[0] == AppWrapper1
         assert conf.application_wrappers[1] == AppWrapper2
         assert conf.application_wrappers[2] == AppWrapper3
         assert conf.application_wrappers[3] == AppWrapper4
+        assert conf.application_wrappers[4] == AppWrapper5
+
+    @raises(TGConfigError)
+    def test_application_wrapper_blocked_after_milestone(self):
+        class AppWrapper1:
+            pass
+        class AppWrapper2:
+            pass
+
+        conf = AppConfig(minimal=True)
+        conf.register_wrapper(AppWrapper1)
+        milestones.environment_loaded.reach()
+        conf.register_wrapper(AppWrapper2)
 
     def test_wrap_app(self):
         class RootController(TGController):
@@ -703,7 +808,7 @@ class TestAppConfig:
         renderers = self.config.renderers
         self.config.renderers = ['unknwon']
         try:
-            self.config.setup_renderers()
+            self.config._setup_renderers()
         except TGConfigError:
             self.config.renderers = renderers
         else:
@@ -988,6 +1093,9 @@ class TestAppConfig:
         assert tw.api.resources.registry.ACTIVE_VARIANT == 'min'
 
     def test_config_hooks(self):
+        # Reset milestone so that registered hooks
+        milestones._reset_all()
+
         class RootController(TGController):
             @expose()
             def test(self):
@@ -1011,6 +1119,61 @@ class TestAppConfig:
         assert 'before_config' in visited_hooks
         assert 'after_config' in visited_hooks
 
+    def test_controller_hooks_with_value(self):
+        # Reset milestone so that registered hooks
+        milestones._reset_all()
+
+        class RootController(TGController):
+            @expose()
+            def test(self):
+                return tg.hooks.notify_with_value('test_hook', 'BO',
+                                                  controller=RootController.test)
+
+        def value_hook(value):
+            return value*2
+
+        tg.hooks.register('test_hook', value_hook, controller=RootController.test)
+
+        conf = AppConfig(minimal=True, root_controller=RootController())
+        app = conf.make_wsgi_app()
+        app = TestApp(app)
+
+        resp = app.get('/test')
+        assert 'BOBO' in resp, resp
+
+    @raises(TGConfigError)
+    def test_config_hooks_startup_on_controller(self):
+        def f():
+            pass
+
+        tg.hooks.register('startup', None, controller=f)
+
+    @raises(TGConfigError)
+    def test_config_hooks_shutdown_on_controller(self):
+        def f():
+            pass
+
+        tg.hooks.register('shutdown', None, controller=f)
+
+    @raises(TGConfigError)
+    def test_controller_wrapper_using_register(self):
+        milestones.config_ready.reach()
+        tg.hooks.register('controller_wrapper', None)
+
+    @raises(TGConfigError)
+    def test_global_controller_wrapper_after_config_ready(self):
+        milestones.config_ready.reach()
+        tg.hooks.wrap_controller(None)
+
+    @raises(TGConfigError)
+    def test_dedicated_controller_wrapper_after_config_ready(self):
+        milestones.config_ready.reach()
+
+        def f():
+            pass
+
+        tg.hooks.wrap_controller(None, controller=f)
+
     def test_error_middleware_disabled_with_optimize(self):
         class RootController(TGController):
             @expose()
