@@ -4,6 +4,7 @@ import tg
 import tests
 from tg.util import DottedFileNameFinder
 from tg.configuration import AppConfig
+from tg.configuration import milestones
 from tg._compat import PY3
 
 
@@ -47,12 +48,17 @@ class TestConfig(AppConfig):
         g = tg.config['tg.app_globals']
         g.dotted_filename_finder = DottedFileNameFinder()
 
-def app_from_config(base_config, deployment_config=None):
+def app_from_config(base_config, deployment_config=None, reset_milestones=True):
     if not deployment_config:
         deployment_config = {'debug': 'true',
                              'error_email_from': 'paste@localhost',
                              'smtp_server': 'localhost'}
 
+    # Reset milestones so that they can be reached again
+    # on next configuration initialization
+    if reset_milestones:
+        milestones._reset_all()
+
     env_loader = base_config.make_load_environment()
     app_maker = base_config.setup_tg_wsgi_app(env_loader)
     app = TestApp(app_maker(deployment_config, full_stack=True))
