@@ -4,12 +4,12 @@ UNDEFINED = runtime.UNDEFINED
 __M_dict_builtin = dict
 __M_locals_builtin = locals
 _magic_number = 8
-_modified_time = 1334006727.9462399
+_modified_time = 1348162811.51627
 _enable_loop = True
-_template_filename = '/home/marvin/src/tg2/tests/test_stack/rendering/templates/mako_noop.mak'
+_template_filename = '/home/amol/tg/turbogears2-tg2-py3/tests/test_stack/rendering/templates/mako_noop.mak'
 _template_uri = 'mako_noop.mak'
 _source_encoding = 'utf-8'
-from webhelpers.html import escape
+from markupsafe import escape_silent as escape
 _exports = []
 
 
@@ -19,7 +19,7 @@ def render_body(context,**pageargs):
         __M_locals = __M_dict_builtin(pageargs=pageargs)
         __M_writer = context.writer()
         # SOURCE LINE 2
-        __M_writer(u'\n<p>This is the mako index page</p>')
+        __M_writer('\n<p>This is the mako index page</p>')
         return ''
     finally:
         context.caller_stack._pop_frame()
