@@ -11,28 +11,25 @@ this writing).
 
 from unittest import TestCase
 from shutil import rmtree
-from urllib import unquote
 import os
 
-from paste import httpexceptions
-from paste.registry import RegistryManager
+from tg._compat import url_unquote
+
+from tg.support.registry import RegistryManager
 from webob import Response, Request
 from webtest import TestApp
-from beaker.middleware import CacheMiddleware, SessionMiddleware
 
-from tg import request, response, expose, require, allow_only
+from tg import request, response, expose, require
 from tg.controllers import TGController, WSGIAppController, RestController
-import pylons
-from pylons import tmpl_context
-from pylons.controllers.util import abort
-from pylons.util import ContextObj, PylonsContext
-from pylons.testutil import ControllerWrap, SetupCacheGlobal
+from tg.controllers.util import abort
+from tg.wsgiapp import ContextObj, TGApp
+from tg.support.middlewares import CacheMiddleware, SessionMiddleware, StatusCodeRedirect
+
+from .baseutils import ControllerWrap, FakeRoutes, default_config
 
-from repoze.who.plugins.auth_tkt import AuthTktCookiePlugin
 from tg.configuration.auth import setup_auth, TGAuthMetadata
 from tg.predicates import is_user, not_anonymous
 
-from pylons.middleware import StatusCodeRedirect
 from tg.error import ErrorHandler
 
 #{ AUT's setup
@@ -47,8 +44,11 @@ rmtree(session_dir, ignore_errors=True)
 def make_app(controller_klass, environ={}, with_errors=False):
     """Creates a ``TestApp`` instance."""
     # The basic middleware:
-    app = ControllerWrap(controller_klass)
-    app = SetupCacheGlobal(app, environ, setup_cache=True, setup_session=True)
+    app = TGApp(config=default_config)
+    app.controller_classes['root'] = ControllerWrap(controller_klass)
+
+    app = FakeRoutes(app)
+    
     if with_errors:
         app = ErrorHandler(app, {}, debug=False)
         app = StatusCodeRedirect(app, [403, 404, 500])
@@ -57,6 +57,7 @@ def make_app(controller_klass, environ={}, with_errors=False):
     app = CacheMiddleware(app, {}, data_dir=os.path.join(data_dir, 'cache'))
 
     # Setting repoze.who up:
+    from repoze.who.plugins.auth_tkt import AuthTktCookiePlugin
     cookie = AuthTktCookiePlugin('secret', 'authtkt')
     identifiers = [('cookie', cookie)]
 
@@ -64,7 +65,6 @@ def make_app(controller_klass, environ={}, with_errors=False):
                      identifiers=identifiers, skip_authentication=True,
                      authenticators=[], challengers=[])
 
-    app = httpexceptions.make_middleware(app)
     return TestApp(app)
 
 
@@ -117,7 +117,20 @@ class ControlPanel(TGController):
     def add_user(self, user_name):
         return "%s was just registered" % user_name
 
+class CustomAllowOnly(TGController):
+    class something(object):
+        def check_authorization(self, env):
+            from tg.controllers.decoratedcontroller import NotAuthorizedError
+            raise NotAuthorizedError()
+
+    @expose()
+    def index(self):
+        return 'HI'
+
+    allow_only = something()
+
 class RootController(TGController):
+    custom_allow = CustomAllowOnly()
 
     cp = ControlPanel()
 
@@ -154,26 +167,6 @@ class ControllerWithAllowOnlyAttributeAndAuthzDenialHandler(TGController):
         abort(402)
 
 
-class ControllerWithAllowOnlyDecoratorAndAuthzDenialHandler(TGController):
-    """
-    Mock TG2 protected controller using the @allow_only decorator, but also
-    using ._failed_authorization()
-
-    """
-
-    @expose()
-    def index(self):
-        return 'Welcome back, foobar!'
-
-    @classmethod
-    def _failed_authorization(self, reason):
-        # Pay first!
-        abort(402)
-
-ControllerWithAllowOnlyDecoratorAndAuthzDenialHandler = allow_only(
-    is_user('foobar'))(ControllerWithAllowOnlyDecoratorAndAuthzDenialHandler)
-
-
 #{ The tests themselves
 
 
@@ -186,19 +179,11 @@ class BaseIntegrationTests(TestCase):
         # Creating the session dir:
         if not os.path.exists(session_dir):
             os.makedirs(session_dir)
+
         # Setting TG2 up:
-        c = ContextObj()
-        py_obj = PylonsContext()
-        py_obj.c = c
-        py_obj.request = py_obj.response = None
-        environ = {'pylons.routes_dict': dict(action='index'),
-                   'pylons.pylons': py_obj}
-        pylons.tmpl_context._push_object(c)
-        # Finally, the app:
-        self.app = make_app(self.controller, environ)
+        self.app = make_app(self.controller, {})
 
     def tearDown(self):
-        tmpl_context._pop_object()
         # Removing the session dir:
         rmtree(session_dir, ignore_errors=True)
 
@@ -208,7 +193,7 @@ class BaseIntegrationTests(TestCase):
 
         """
         assert 'webflash' in response.cookies_set, "Such no WebFlash cookie"
-        flash = unquote(response.cookies_set['webflash'])
+        flash = url_unquote(response.cookies_set['webflash'])
         for msg in expected_messages:
             msg = '"%s"' % msg
             assert msg in flash, 'Message %s not in flash: %s' % (msg, flash)
@@ -217,38 +202,42 @@ class BaseIntegrationTests(TestCase):
 class TestRequire(BaseIntegrationTests):
     """Test case for the @require decorator"""
 
+    def test_authz_custom_allow_only(self):
+        #environ = {'REMOTE_USER': 'developer'}
+        resp = self.app.get('/custom_allow', extra_environ={}, status=401)
+
     def test_authz_granted_in_root_controller(self):
         environ = {'REMOTE_USER': 'developer'}
         resp = self.app.get('/commit', extra_environ=environ, status=200)
-        self.assertEqual("you can commit", resp.body)
+        self.assertEqual("you can commit", resp.body.decode('utf-8'))
 
     def test_authz_denied_in_root_controller(self):
         # As an anonymous user:
         resp = self.app.get('/commit', status=401)
-        assert "you can commit" not in resp.body
+        assert "you can commit" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"developer\"')
         # As an authenticated user:
         environ = {'REMOTE_USER': 'foobar'}
         resp = self.app.get('/commit', extra_environ=environ, status=403)
-        assert "you can commit" not in resp.body
+        assert "you can commit" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"developer\"')
 
     def test_authz_granted_in_sub_controller(self):
         environ = {'REMOTE_USER': 'admin'}
         resp = self.app.get('/cp/add_user/foo', extra_environ=environ,
                             status=200)
-        self.assertEqual("foo was just registered", resp.body)
+        self.assertEqual("foo was just registered", resp.body.decode('utf-8'))
 
     def test_authz_denied_in_sub_controller(self):
         # As an anonymous user:
         resp = self.app.get('/cp/add_user/foo', status=401)
-        assert "was just registered" not in resp.body
+        assert "was just registered" not in resp.body.decode('utf-8')
         self._check_flash(resp, NOT_AUTHENTICATED)
         # As an authenticated user:
         environ = {'REMOTE_USER': 'foobar'}
         resp = self.app.get('/cp/add_user/foo', extra_environ=environ,
                             status=403)
-        assert "was just registered" not in resp.body
+        assert "was just registered" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"admin\"')
 
     def test_smart_auth_json(self):
@@ -257,12 +246,13 @@ class TestRequire(BaseIntegrationTests):
                 'REMOTE_USER': 'foobar'}
         gooduser = {'accept': 'application/json',
                 'REMOTE_USER': 'developer'}
+
         resp = self.app.get('/smartabort', extra_environ=nouser, status=401)
         assert resp.status == '401 Unauthorized', 'Expected 401, got %s' % (resp.status)
         resp = self.app.get('/smartabort', extra_environ=baduser, status=403)
         assert resp.status == '403 Forbidden', 'Expected 403, got %s' % (resp.status)
         resp = self.app.get('/smartabort.json', extra_environ=gooduser, status=200)
-        assert resp.status == '200 OK', 'Expected 200, got %s' % (resp.body)    
+        assert resp.status == '200 OK', 'Expected 200, got %s' % (resp.body)
 
 
 class TestAllowOnlyDecoratorInSubController(BaseIntegrationTests):
@@ -271,44 +261,24 @@ class TestAllowOnlyDecoratorInSubController(BaseIntegrationTests):
     def test_authz_granted_without_require(self):
         environ = {'REMOTE_USER': 'someone'}
         resp = self.app.get('/cp/', extra_environ=environ, status=200)
-        self.assertEqual("you are in the panel", resp.body)
+        self.assertEqual("you are in the panel", resp.body.decode('utf-8'))
 
     def test_authz_denied_without_require(self):
         resp = self.app.get('/cp/', status=401)
-        assert "you are in the panel" not in resp.body
+        assert "you are in the panel" not in resp.body.decode('utf-8')
         self._check_flash(resp, NOT_AUTHENTICATED)
 
     def test_authz_granted_with_require(self):
         environ = {'REMOTE_USER': 'admin'}
         resp = self.app.get('/cp/add_user/foo', extra_environ=environ,
                             status=200)
-        self.assertEqual("foo was just registered", resp.body)
+        self.assertEqual("foo was just registered", resp.body.decode('utf-8'))
 
     def test_authz_denied_with_require(self):
         resp = self.app.get('/cp/add_user/foo', status=401)
-        assert "was just registered" not in resp.body
+        assert "was just registered" not in resp.body.decode('utf-8')
         self._check_flash(resp, NOT_AUTHENTICATED)
 
-
-class _TestAllowOnlyDecoratorAndDefaultAuthzDenialHandler(BaseIntegrationTests):
-    """
-    Test case for the @allow_only decorator in a controller using
-    _failed_authorization() as its denial handler.
-
-    """
-
-    controller = ControllerWithAllowOnlyDecoratorAndAuthzDenialHandler
-
-    def test_authz_granted(self):
-        environ = {'REMOTE_USER': 'foobar'}
-        resp = self.app.get('/', extra_environ=environ, status=200)
-        self.assertEqual("Welcome back, foobar!", resp.body)
-
-    def test_authz_denied(self):
-        resp = self.app.get('/', status=402)
-        assert "Welcome back" not in resp.body
-
-
 class TestAllowOnlyAttributeInSubController(BaseIntegrationTests):
     """Test case for the .allow_only attribute in a sub-controller"""
 
@@ -317,37 +287,36 @@ class TestAllowOnlyAttributeInSubController(BaseIntegrationTests):
     def test_authz_granted_without_require(self):
         environ = {'REMOTE_USER': 'hiring-manager'}
         resp = self.app.get('/hr/', extra_environ=environ, status=200)
-        self.assertEqual("you can manage Human Resources", resp.body)
+        self.assertEqual("you can manage Human Resources", resp.body.decode('utf-8'))
 
     def test_authz_denied_without_require(self):
         # As an anonymous user:
         resp = self.app.get('/hr/', status=401)
-        assert "you can manage Human Resources" not in resp.body
+        assert "you can manage Human Resources" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must have been authenticated')
         # As an authenticated user:
         environ = {'REMOTE_USER': 'someone'}
         resp = self.app.get('/hr/', extra_environ = environ, status=403)
-        assert "you can manage Human Resources" not in resp.body
+        assert "you can manage Human Resources" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"hiring-manager\"')
 
     def test_authz_granted_with_require(self):
         environ = {'REMOTE_USER': 'hiring-manager'}
         resp = self.app.get('/hr/hire/gustavo', extra_environ=environ,
                             status=200)
-        self.assertEqual("gustavo was just hired", resp.body)
+        self.assertEqual("gustavo was just hired", resp.body.decode('utf-8'))
 
     def test_authz_denied_with_require(self):
         # As an anonymous user:
         resp = self.app.get('/hr/hire/gustavo', status=401)
-        assert "was just hired" not in resp.body
+        assert "was just hired" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must have been authenticated')
         # As an authenticated user:
         environ = {'REMOTE_USER': 'someone'}
         resp = self.app.get('/hr/hire/gustavo', extra_environ = environ, status=403)
-        assert "was just hired" not in resp.body
+        assert "was just hired" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"hiring-manager\"')
 
-
 class TestAllowOnlyAttributeAndDefaultAuthzDenialHandler(BaseIntegrationTests):
     """
     Test case for the .allow_only attribute in a controller using
@@ -360,12 +329,11 @@ class TestAllowOnlyAttributeAndDefaultAuthzDenialHandler(BaseIntegrationTests):
     def test_authz_granted(self):
         environ = {'REMOTE_USER': 'foobar'}
         resp = self.app.get('/', extra_environ=environ, status=200)
-        self.assertEqual("Welcome back, foobar!", resp.body)
+        self.assertEqual("Welcome back, foobar!", resp.body.decode('utf-8'))
 
     def test_authz_denied(self):
         resp = self.app.get('/', status=402)
-        assert "Welcome back" not in resp.body
-
+        assert "Welcome back" not in resp.body.decode('utf-8')
 
 class TestAppWideAuthzWithAllowOnlyDecorator(BaseIntegrationTests):
     """Test case for application-wide authz with the @allow_only decorator"""
@@ -375,22 +343,22 @@ class TestAppWideAuthzWithAllowOnlyDecorator(BaseIntegrationTests):
     def test_authz_granted_without_require(self):
         environ = {'REMOTE_USER': 'someone'}
         resp = self.app.get('/', extra_environ=environ, status=200)
-        self.assertEqual("you are in the panel", resp.body)
+        self.assertEqual("you are in the panel", resp.body.decode('utf-8'))
 
     def test_authz_denied_without_require(self):
         resp = self.app.get('/', status=401)
-        assert "you are in the panel" not in resp.body
+        assert "you are in the panel" not in resp.body.decode('utf-8')
         self._check_flash(resp, NOT_AUTHENTICATED)
 
     def test_authz_granted_with_require(self):
         environ = {'REMOTE_USER': 'admin'}
         resp = self.app.get('/add_user/foo', extra_environ=environ,
                             status=200)
-        self.assertEqual("foo was just registered", resp.body)
+        self.assertEqual("foo was just registered", resp.body.decode('utf-8'))
 
     def test_authz_denied_with_require(self):
         resp = self.app.get('/add_user/foo', status=401)
-        assert "was just registered" not in resp.body
+        assert "was just registered" not in resp.body.decode('utf-8')
         self._check_flash(resp, NOT_AUTHENTICATED)
 
 
@@ -402,34 +370,34 @@ class TestAppWideAuthzWithAllowOnlyAttribute(BaseIntegrationTests):
     def test_authz_granted_without_require(self):
         environ = {'REMOTE_USER': 'hiring-manager'}
         resp = self.app.get('/', extra_environ=environ, status=200)
-        self.assertEqual("you can manage Human Resources", resp.body)
+        self.assertEqual("you can manage Human Resources", resp.body.decode('utf-8'))
 
     def test_authz_denied_without_require(self):
         # As an anonymous user:
         resp = self.app.get('/', status=401)
-        assert "you can manage Human Resources" not in resp.body
+        assert "you can manage Human Resources" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"hiring-manager\"')
         # As an authenticated user:
         environ = {'REMOTE_USER': 'someone'}
         resp = self.app.get('/', extra_environ = environ, status=403)
-        assert "you can manage Human Resources" not in resp.body
+        assert "you can manage Human Resources" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"hiring-manager\"')
 
     def test_authz_granted_with_require(self):
         environ = {'REMOTE_USER': 'hiring-manager'}
         resp = self.app.get('/hire/gustavo', extra_environ=environ,
                             status=200)
-        self.assertEqual("gustavo was just hired", resp.body)
+        self.assertEqual("gustavo was just hired", resp.body.decode('utf-8'))
 
     def test_authz_denied_with_require(self):
         # As an anonymous user:
         resp = self.app.get('/hire/gustavo', status=401)
-        assert "was just hired" not in resp.body
+        assert "was just hired" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"hiring-manager\"')
         # As an authenticated user:
         environ = {'REMOTE_USER': 'someone'}
         resp = self.app.get('/hire/gustavo', extra_environ = environ, status=403)
-        assert "was just hired" not in resp.body
+        assert "was just hired" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"hiring-manager\"')
 
 
@@ -440,17 +408,17 @@ class TestProtectedRESTContoller(BaseIntegrationTests):
         environ = {'REMOTE_USER': 'gustavo'}
         resp = self.app.get('/rest/new', extra_environ=environ,
                             status=200)
-        self.assertEqual("new here", resp.body)
+        self.assertEqual("new here", resp.body.decode('utf-8'))
 
     def test_authz_denied(self):
         # As an anonymous user:
         resp = self.app.get('/rest/new', status=401)
-        assert "new here" not in resp.body
+        assert "new here" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"gustavo\"')
         # As an authenticated user:
         environ = {'REMOTE_USER': 'non-gustavo'}
         resp = self.app.get('/rest/new', extra_environ=environ, status=403)
-        assert "new here" not in resp.body
+        assert "new here" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"gustavo\"')
 
 
@@ -461,18 +429,18 @@ class TestProtectedWSGIApplication(BaseIntegrationTests):
         environ = {'REMOTE_USER': 'gustavo'}
         resp = self.app.get('/mounted_app/da-path', extra_environ=environ,
                             status=200)
-        self.assertEqual("Hello from /mounted_app/da-path", resp.body)
+        self.assertEqual("Hello from /mounted_app/da-path", resp.body.decode('utf-8'))
 
     def test_authz_denied(self):
         # As an anonymous user:
         resp = self.app.get('/mounted_app/da-path', status=401)
-        assert "Hello from /mounted_app/" not in resp.body
+        assert "Hello from /mounted_app/" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"gustavo\"')
         # As an authenticated user:
         environ = {'REMOTE_USER': 'non-gustavo'}
         resp = self.app.get('/mounted_app/da-path', extra_environ=environ,
                             status=403)
-        assert "Hello from /mounted_app/" not in resp.body
+        assert "Hello from /mounted_app/" not in resp.body.decode('utf-8')
         self._check_flash(resp, r'The current user must be \"gustavo\"')
 
 class ErrorController(object):
@@ -491,15 +459,9 @@ class TestLoggedErrorTGController(BaseIntegrationTests):
     def setUp(self):
         if not os.path.exists(session_dir):
             os.makedirs(session_dir)
+
         # Setting TG2 up:
-        c = ContextObj()
-        py_obj = PylonsContext()
-        py_obj.c = c
-        py_obj.request = py_obj.response = None
-        environ = {'pylons.routes_dict': dict(action='index'),
-                   'pylons.pylons': py_obj}
-        pylons.tmpl_context._push_object(c)
-        self.app = make_app(DefaultLessTGController, environ, with_errors=True)
+        self.app = make_app(DefaultLessTGController, {}, with_errors=True)
 
     def test_logged_index(self):
         resp = self.app.get('/index', extra_environ={'REMOTE_USER': 'gustavo'}, expect_errors=True)
