@@ -6,7 +6,7 @@ import logging
 import warnings
 from copy import copy, deepcopy
 import mimetypes
-from collections import MutableMapping as DictMixin
+from collections import MutableMapping as DictMixin, deque
 
 from tg.i18n import ugettext, ungettext, get_lang
 
@@ -19,10 +19,11 @@ from tg.request_local import config as reqlocal_config
 import tg
 from tg.configuration.utils import coerce_config
 from tg.util import Bunch, get_partial_dict, DottedFileNameFinder, call_controller
+from tg.configuration import milestones
+from tg.configuration.utils import TGConfigError
 
 log = logging.getLogger(__name__)
 
-class TGConfigError(Exception):pass
 
 class DispatchingConfigWrapper(DictMixin):
     """Wrapper for the Dispatching configuration.
@@ -167,6 +168,7 @@ class AppConfig(Bunch):
         self.prefer_toscawidgets2 = False
         self.use_dotted_templatenames = not minimal
         self.handle_error_page = not minimal
+        self.registry_streaming = True
 
         self.use_sessions = not minimal
         self.i18n_enabled = not minimal
@@ -177,7 +179,10 @@ class AppConfig(Bunch):
         self.call_on_shutdown = []
         self.controller_caller = call_controller
         self.controller_wrappers = []
+        self.dedicated_controller_wrappers = {}
         self.application_wrappers = []
+        self.application_wrappers_dependencies = {False: [],
+                                                  None: []}
         self.hooks = dict(before_validate=[],
                           before_call=[],
                           before_render=[],
@@ -186,6 +191,7 @@ class AppConfig(Bunch):
                           after_render_call=[],
                           before_config=[],
                           after_config=[])
+
         # The codes TG should display an error page for. All other HTTP errors are
         # sent to the client or left for some middleware above us to handle
         self.handle_status_codes = [403, 404]
@@ -219,41 +225,14 @@ class AppConfig(Bunch):
         return root_controller_module
 
     def register_hook(self, hook_name, func):
-        """Registers a TurboGears hook or controller wrapper.
+        warnings.warn("AppConfig.register_hook is deprecated, "
+                      "please use tg.hooks.register and "
+                      "tg.hooks.wrap_controller instead", DeprecationWarning)
 
-        Given an hook name and a function it registers the provided
-        function for that role. For a complete list of hooks
-        provided by default have a look at :ref:`hooks_and_events`.
-
-        If the provided hook name is ``controller_wrapper`` the
-        function is registered as a controller wrapper.
-        Controller Wrappers are much like a **decorator** applied to
-        every controller.
-        They receive :class:`tg.configuration.AppConfig` instance
-        as an argument and the next handler in chain and are expected
-        to return a new handler that performs whatever it requires
-        and then calls the next handler.
-
-        A simple example for a controller wrapper is a simple logging wrapper::
-
-            def controller_wrapper(app_config, caller):
-                def call(*args, **kw):
-                    try:
-                        print 'Before handler!'
-                        return caller(*args, **kw)
-                    finally:
-                        print 'After Handler!'
-                return call
-
-        """
-        if hook_name == 'startup':
-            self.call_on_startup.append(func)
-        elif hook_name == 'shutdown':
-            self.call_on_shutdown.append(func)
-        elif hook_name == 'controller_wrapper':
-            self.controller_wrappers.append(func)
+        if hook_name == 'controller_wrapper':
+            tg.hooks.wrap_controller(func)
         else:
-            self.hooks.setdefault(hook_name, []).append(func)
+            tg.hooks.register(hook_name, func)
 
     def register_wrapper(self, wrapper, after=None):
         """Registers a TurboGears application wrapper.
@@ -277,24 +256,30 @@ class AppConfig(Bunch):
 
                 def __call__(self, environ, context):
                     print 'Going to run %s' % context.request.path
-                    return self.handler(*args, **kw)
+                    return self.handler(environ, context)
         """
-        wrappers = self.application_wrappers
+        if milestones.environment_loaded.reached:
+            # We must block registering wrappers if milestone passed, this is because
+            # wrappers are consumed by TGApp constructor, and all the hooks available
+            # after the milestone and that could register new wrappers are actually
+            # called after TGApp constructors and so the wrappers wouldn't be applied.
+            raise TGConfigError('Cannot register application wrappers after application '
+                                'environment has already been loaded')
 
-        if after is False:
-            append_index = 0
-        else:
-            append_index = len(wrappers)
+        self.application_wrappers_dependencies.setdefault(after, []).append(wrapper)
+        milestones.environment_loaded.register(self._order_wrappers)
 
-        if after:
-            for idx, current_wrapper in enumerate(wrappers):
-                if current_wrapper == after:
-                    append_index = idx + 1
-                    break
+    def _order_wrappers(self):
+        visit_queue = deque([False, None])
+        while visit_queue:
+            current = visit_queue.popleft()
+            if current not in (False, None):
+                self.application_wrappers.append(current)
 
-        wrappers.insert(append_index, wrapper)
+            dependant_wrappers = self.application_wrappers_dependencies.pop(current, [])
+            visit_queue.extendleft(reversed(dependant_wrappers))
 
-    def setup_startup_and_shutdown(self):
+    def _setup_startup_and_shutdown(self):
         for cmd in self.call_on_startup:
             if callable(cmd):
                 try:
@@ -310,7 +295,7 @@ class AppConfig(Bunch):
             else:
                 log.warn("Unable to register %s for shutdown" % cmd )
 
-    def setup_package_paths(self):
+    def _setup_package_paths(self):
         root = os.path.dirname(os.path.abspath(self.package.__file__))
         # The default paths:
         paths = Bunch(root=root,
@@ -322,7 +307,7 @@ class AppConfig(Bunch):
         paths.update(self.paths)
         self.paths = paths
 
-    def init_config(self, global_conf, app_conf):
+    def _init_config(self, global_conf, app_conf):
         """Initialize the config object.
 
         Besides basic initialization, this method copies all the values
@@ -401,6 +386,8 @@ class AppConfig(Bunch):
         if not conf['paths']['static_files']:
             self.serve_static = False
 
+        self._configure_renderers()
+
         config.update(self)
 
         #see http://trac.turbogears.org/ticket/2247
@@ -410,6 +397,17 @@ class AppConfig(Bunch):
             config['tg.strict_tmpl_context'] = False
 
         self.after_init_config()
+        milestones.config_ready.reach()
+
+    def _configure_renderers(self):
+        """Provides default configurations for renderers"""
+        if not 'json' in self.renderers:
+            self.renderers.append('json')
+
+        if self.default_renderer not in self.renderers:
+            first_renderer = self.renderers[0]
+            log.warn('Default renderer not in renders, automatically switching to %s' % first_renderer)
+            self.default_renderer = first_renderer
 
     def after_init_config(self):
         """
@@ -461,6 +459,9 @@ class AppConfig(Bunch):
             base_config = MyAppConfig()
 
         """
+        if not self.enable_routes:
+            return None
+
         from routes import Mapper
 
         map = Mapper(directory=config['paths']['controllers'],
@@ -470,6 +471,7 @@ class AppConfig(Bunch):
         map.connect('*url', controller='root', action='routes_placeholder')
 
         config['routes.map'] = map
+        return map
 
     def setup_helpers_and_globals(self):
         """Add helpers and globals objects to the config.
@@ -835,20 +837,27 @@ class AppConfig(Bunch):
             self.sa_auth.setdefault('form_plugin', None)
             self.sa_auth.setdefault('cookie_secret', config['beaker.session.secret'])
 
-    def setup_controller_wrappers(self):
-        controller_caller = config.get('controller_caller')
+    def _setup_controller_wrappers(self):
+        base_controller_caller = config.get('controller_caller')
+
+        controller_caller = base_controller_caller
         for wrapper in self.get('controller_wrappers', []):
             controller_caller = wrapper(self, controller_caller)
         config['controller_caller'] = controller_caller
 
-    def setup_renderers(self):
-        if not 'json' in self.renderers: self.renderers.append('json')
-
-        if self.default_renderer not in self.renderers:
-            first_renderer = self.renderers[0]
-            log.warn('Default renderer not in renders, automatically switching to %s' % first_renderer)
-            self.default_renderer = first_renderer
+        dedicated_wrappers = config.get('dedicated_controller_wrappers', {})
+        for wrapped_controller in dedicated_wrappers:
+            controller_caller = base_controller_caller
+            wrappers = dedicated_wrappers[wrapped_controller]
+            # Apply custom wrappers for controller
+            for wrapper in wrappers:
+                controller_caller = wrapper(self, controller_caller)
+            # Apply generic wrappers for application
+            for wrapper in self.get('controller_wrappers', []):
+                controller_caller = wrapper(self, controller_caller)
+            dedicated_wrappers[wrapped_controller] = controller_caller
 
+    def _setup_renderers(self):
         for renderer in self.renderers[:]:
             setup = getattr(self, 'setup_%s_renderer'%renderer, None)
             if setup:
@@ -859,6 +868,8 @@ class AppConfig(Bunch):
             else:
                 raise TGConfigError('This configuration object does not support the %s renderer' % renderer)
 
+        milestones.renderers_ready.reach()
+
     def make_load_environment(self):
         """Return a load_environment function.
 
@@ -883,25 +894,26 @@ class AppConfig(Bunch):
                 app_package = None
 
             if app_package:
-                self.setup_package_paths()
+                self._setup_package_paths()
 
-            self.init_config(global_conf, app_conf)
+            self._init_config(global_conf, app_conf)
 
             #Registers functions to be called at startup and shutdown
             #from self.call_on_startup and shutdown respectively.
-            self.setup_startup_and_shutdown()
+            self._setup_startup_and_shutdown()
 
-            if self.enable_routes:
-                self.setup_routes()
+            self.setup_routes()
 
             if app_package:
                 self.setup_helpers_and_globals()
 
             self.setup_mimetypes()
             self.setup_auth()
-            self.setup_renderers()
+            self._setup_renderers()
             self.setup_persistence()
 
+            milestones.environment_loaded.reach()
+
         return load_environment
 
     def add_error_middleware(self, global_conf, app):
@@ -1020,6 +1032,8 @@ class AppConfig(Bunch):
             base_config = MyAppConfig()
         """
         if self.enable_routes:
+            warnings.warn("Internal routes support will be deprecated soon, please "
+                          "consider using tgext.routes instead", DeprecationWarning)
             from routes.middleware import RoutesMiddleware
             app = RoutesMiddleware(app, config['routes.map'])
 
@@ -1218,27 +1232,22 @@ class AppConfig(Bunch):
             if load_environment:
                 load_environment(global_conf, app_conf)
 
+            # Apply controller wrappers to controller caller
+            self._setup_controller_wrappers()
+
+            # TODO: This should be moved in configuration phase.
+            # It is here as it requires both the .ini file and AppConfig to be ready
+            avoid_sess_touch = config.get('beaker.session.tg_avoid_touch', 'false')
+            config['beaker.session.tg_avoid_touch'] = asbool(avoid_sess_touch)
+
             app = TGApp()
             if wrap_app:
                 app = wrap_app(app)
 
-            for hook in self.hooks['before_config']:
-                app = hook(app)
-
-            #Apply controller wrappers to controller caller
-            self.setup_controller_wrappers()
-
-            avoid_sess_touch = config.get('beaker.session.tg_avoid_touch', 'false')
-            config['beaker.session.tg_avoid_touch'] = asbool(avoid_sess_touch)
+            app = tg.hooks.notify_with_value('before_config', app, context_config=config)
 
             app = self.add_core_middleware(app)
 
-            if self.use_toscawidgets:
-                app = self.add_tosca_middleware(app)
-
-            if self.use_toscawidgets2:
-                app = self.add_tosca2_middleware(app)
-
             if self.auth_backend:
                 # Skipping authentication if explicitly requested. Used by
                 # repoze.who-testutil:
@@ -1248,6 +1257,21 @@ class AppConfig(Bunch):
             if self.use_transaction_manager:
                 app = self.add_tm_middleware(app)
 
+            # TODO: Middlewares before this point should be converted to App Wrappers.
+            # They provide some basic TG features like AUTH, Caching and transactions
+            # which should be app wrappers to make possible to add wrappers in the
+            # stack before or after them.
+
+            if self.use_toscawidgets:
+                app = self.add_tosca_middleware(app)
+
+            if self.use_toscawidgets2:
+                app = self.add_tosca2_middleware(app)
+
+            # from here on the response is a generator
+            # so any middleware that relies on the response to be
+            # a string needs to be applied before this point.
+
             if self.use_sqlalchemy:
                 if not hasattr(self, 'DBSession'):
                     # If the user hasn't specified a scoped_session, assume
@@ -1277,7 +1301,7 @@ class AppConfig(Bunch):
                 app = self.add_error_middleware(global_conf, app)
 
             # Establish the registry for this application
-            app = RegistryManager(app, streaming=True,
+            app = RegistryManager(app, streaming=config.get('registry_streaming', True),
                                   preserve_exceptions=asbool(global_conf.get('debug')))
 
             # Place the debuggers after the registry so that we
@@ -1295,8 +1319,7 @@ class AppConfig(Bunch):
             if self.serve_static:
                 app = self.add_static_file_middleware(app)
 
-            for hook in self.hooks['after_config']:
-                app = hook(app)
+            app = tg.hooks.notify_with_value('after_config', app, context_config=config)
 
             return app
 
